@@ -179,7 +179,8 @@ void struct_matvec(const int stencil_type,
   int64_t worksets_ext =
       (numInteriorPts + rows_per_team_ext - 1) / rows_per_team_ext;
 
-  KokkosSparse::Impl::SPMV_Struct_Functor<AMatrix, XVector, YVector, 1, false>
+  KokkosSparse::Impl::SPMV_Struct_Functor<execution_space, AMatrix, XVector,
+                                          YVector, 1, false>
       spmv_struct(structure, stencil_type, alpha, A, x, beta, y,
                   rows_per_team_int, rows_per_team_ext);
 
@@ -188,8 +189,10 @@ void struct_matvec(const int stencil_type,
               << ", vector_length=" << vector_length << std::endl;
   }
 
-  spmv_struct.compute_interior(worksets_int, team_size_int, vector_length);
-  spmv_struct.compute_exterior(worksets_ext, team_size_ext, vector_length);
+  spmv_struct.compute_interior(execution_space{}, worksets_int, team_size_int,
+                               vector_length);
+  spmv_struct.compute_exterior(execution_space{}, worksets_ext, team_size_ext,
+                               vector_length);
 
 }  // struct_matvec
 
@@ -210,8 +213,9 @@ void matvec(typename YVector::const_value_type& alpha, const AMatrix& A,
           A.numRows(), A.nnz(), rows_per_thread, team_size, vector_length);
   int64_t worksets = (y.extent(0) + rows_per_team - 1) / rows_per_team;
 
-  KokkosSparse::Impl::SPMV_Functor<AMatrix, XVector, YVector, 1, false> func(
-      alpha, A, x, beta, y, rows_per_team);
+  KokkosSparse::Impl::SPMV_Functor<execution_space, AMatrix, XVector, YVector,
+                                   1, false>
+      func(alpha, A, x, beta, y, rows_per_team);
 
   if (print_lp) {
     std::cout << "worksets=" << worksets << ", team_size=" << team_size
@@ -577,7 +581,9 @@ int main(int argc, char** argv) {
       const double alpha = 1.0, beta = 1.0;
       size_t bufferSize = 0;
       void* dBuffer     = NULL;
-#if CUSPARSE_VERSION >= 11201
+
+// CUSPARSE_MM_ALG_DEFAULT was deprecated in CUDA 11.2.1 a.k.a cuSPARSE 11.4.0
+#if CUSPARSE_VERSION >= 11400
       cusparseSpMVAlg_t alg = CUSPARSE_SPMV_ALG_DEFAULT;
 #else
       cusparseSpMVAlg_t alg = CUSPARSE_MV_ALG_DEFAULT;
