@@ -58,26 +58,34 @@ void Refinement::declare_refinement_fields()
   stk::mesh::put_field_on_mesh(*myOriginatingProcForParentElementField, myMeta.universal_part(), 1, nullptr); // needed everywhere for restart, otherwise could be parent_part
 }
 
-Refinement::Refinement(stk::mesh::MetaData & meta, stk::mesh::Part * activePart, const bool force64Bit, const bool assert32Bit)
-  : myMeta(meta),
-    myForce64Bit(force64Bit),
-    myAssert32Bit(assert32Bit),
-    myNodeRefiner(force64Bit, assert32Bit),
-    myEntityIdPool(meta),
-    myActivePart(activePart)
+Refinement::Refinement(stk::mesh::MetaData & meta,
+    stk::mesh::Part * activePart,
+    const bool force64Bit,
+    const bool assert32Bit,
+    stk::diag::Timer & parentTimer)
+    : myMeta(meta),
+      myForce64Bit(force64Bit),
+      myAssert32Bit(assert32Bit),
+      myNodeRefiner(force64Bit, assert32Bit),
+      myEntityIdPool(meta),
+      myActivePart(activePart),
+      refineTimer(parentTimer),
+      unrefineTimer(parentTimer),
+      myFixPartsandOwnersTimer("Fix Parts and Owners", parentTimer)
 {
   myCoordsField = static_cast<const stk::mesh::Field<double>*>(myMeta.coordinate_field());
   declare_refinement_parts();
   declare_refinement_fields();
 }
 
-Refinement::Refinement(stk::mesh::MetaData & meta, stk::mesh::Part * activePart)
-  : Refinement(meta, activePart, false, false)
+Refinement::Refinement(
+    stk::mesh::MetaData & meta, stk::mesh::Part * activePart, stk::diag::Timer & parentTimer)
+    : Refinement(meta, activePart, false, false, parentTimer)
 {
 }
 
-Refinement::Refinement(stk::mesh::MetaData & meta)
-  : Refinement(meta, nullptr, false, false)
+Refinement::Refinement(stk::mesh::MetaData & meta, stk::diag::Timer & parentTimer)
+  : Refinement(meta, nullptr, false, false, parentTimer)
 {
 }
 
@@ -906,28 +914,41 @@ void Refinement::update_element_rebalance_weights_incorporating_parallel_owner_c
 void Refinement::create_refined_nodes_elements_and_sides(const EdgeMarkerInterface & edgeMarker)
 {
   stk::mesh::BulkData & mesh = myMeta.mesh_bulk_data();
-
-  mesh.modification_begin();
-
-  destroy_custom_ghostings();
-
-  myNodeRefiner.create_refined_edge_nodes(mesh, get_parts_for_new_refined_edge_nodes(), myRefinedEdgeNodeParentIdsField);
-
-  const std::vector<BucketData> bucketsData = get_buckets_data_for_candidate_elements_to_refine(edgeMarker);
-  const size_t numNewElements = count_new_child_elements(edgeMarker, bucketsData);
-  myEntityIdPool.reserve(stk::topology::ELEMENT_RANK, numNewElements, myAssert32Bit, myForce64Bit);
-
-  std::vector<stk::mesh::Entity> elementsToDelete;
   std::vector<SideDescription> sideRequests;
-  refine_elements_with_refined_edges_and_store_sides_to_create(edgeMarker, bucketsData, sideRequests, elementsToDelete);
-  stk::mesh::destroy_elements_no_mod_cycle(mesh, elementsToDelete, mesh.mesh_meta_data().universal_part());
 
-  mesh.modification_end();
+  {
+    mesh.modification_begin();
 
+    // Mesh modification cycle
+    destroy_custom_ghostings();
+
+    myNodeRefiner.create_refined_edge_nodes(
+        mesh, get_parts_for_new_refined_edge_nodes(), myRefinedEdgeNodeParentIdsField);
+
+    const std::vector<BucketData> bucketsData =
+        get_buckets_data_for_candidate_elements_to_refine(edgeMarker);
+    const size_t numNewElements = count_new_child_elements(edgeMarker, bucketsData);
+    myEntityIdPool.reserve(
+        stk::topology::ELEMENT_RANK, numNewElements, myAssert32Bit, myForce64Bit);
+
+    std::vector<stk::mesh::Entity> elementsToDelete;
+    refine_elements_with_refined_edges_and_store_sides_to_create(
+        edgeMarker, bucketsData, sideRequests, elementsToDelete);
+    stk::mesh::destroy_elements_no_mod_cycle(
+        mesh, elementsToDelete, mesh.mesh_meta_data().universal_part());
+
+    mesh.modification_end();
+  }
+
+  //timer batch create sides
   if(stk::is_true_on_any_proc(mesh.parallel(), !sideRequests.empty()))
+  {
     batch_create_sides(mesh, sideRequests);
+  }
 
-  myNodeRefiner.prolong_refined_edge_nodes(mesh);
+  {
+    myNodeRefiner.prolong_refined_edge_nodes(mesh);
+  }
 }
 
 void Refinement::create_another_layer_of_refined_elements_and_sides_to_eliminate_hanging_nodes(const EdgeMarkerInterface & edgeMarker)
@@ -1029,81 +1050,145 @@ void Refinement::destroy_custom_ghostings()
   krino::destroy_custom_ghostings(myMeta.mesh_bulk_data());
 }
 
-bool Refinement::do_unrefinement(const EdgeMarkerInterface & edgeMarker)
+void Refinement::finalize()
 {
+  stk::diag::TimeBlock timer_(myFixPartsandOwnersTimer);
+  stk::mesh::BulkData & mesh = myMeta.mesh_bulk_data();
+  activate_selected_entities_touching_active_elements(
+      mesh, myMeta.side_rank(), myMeta.universal_part(), *myActivePart);
+  fix_node_owners_to_assure_active_owned_element_for_node(mesh, *myActivePart);
+}
+
+bool Refinement::refine_elements(const EdgeMarkerInterface & edgeMarker)
+{
+  stk::diag::TimeBlock timer_(refineTimer.rootTimer);
+
   stk::mesh::BulkData & mesh = myMeta.mesh_bulk_data();
 
-  check_leaf_children_have_parents_on_same_proc();
+  {
+    stk::diag::TimeBlock timer_2(refineTimer.checkLeafChildren);
+    check_leaf_children_have_parents_on_same_proc();
+  }
+
+  {
+    stk::diag::TimeBlock timer_2(refineTimer.findEdgesToRefine);
+    find_edges_to_refine(edgeMarker);
+  }
+  bool didMakeAnyChanges = false;
+
+  const bool haveEdgesToRefineLocally = get_num_edges_to_refine() > 0;
+  if (stk::is_true_on_any_proc(mesh.parallel(), haveEdgesToRefineLocally))
+  {
+    didMakeAnyChanges = true;
+    {
+      stk::diag::TimeBlock timer_2(refineTimer.createRefinedEdges);
+      create_refined_nodes_elements_and_sides(edgeMarker);
+    }
+
+    {
+      stk::diag::TimeBlock timer_2(refineTimer.elminateHangingNodes);
+      create_another_layer_of_refined_elements_and_sides_to_eliminate_hanging_nodes(edgeMarker);
+    }
+
+    {
+      stk::diag::TimeBlock timer_2(refineTimer.prolongNodes);
+      myNodeRefiner.prolong_refined_edge_nodes(mesh);
+    }
+  }
+
+  return didMakeAnyChanges;
+}
+
+bool Refinement::unrefine_elements(const EdgeMarkerInterface & edgeMarker)
+{
+  stk::diag::TimeBlock timer_(unrefineTimer.rootTimer);
+
+  stk::mesh::BulkData & mesh = myMeta.mesh_bulk_data();
+
+  {
+    stk::diag::TimeBlock timer_2(unrefineTimer.checkLeafChildren);
+    check_leaf_children_have_parents_on_same_proc();
+  }
 
   bool didMakeAnyChanges = false;
   if(stk::is_true_on_any_proc(mesh.parallel(), edgeMarker.locally_have_elements_to_unrefine()))
   {
     std::vector<stk::mesh::Entity> childElementsToDeleteForUnrefinement;
     std::vector<stk::mesh::Entity> ownedParentElementsModifiedByUnrefinement;
-    edgeMarker.fill_elements_modified_by_unrefinement(ownedParentElementsModifiedByUnrefinement, childElementsToDeleteForUnrefinement);
+    {
+      stk::diag::TimeBlock timer_2(unrefineTimer.fillElements);
+      edgeMarker.fill_elements_modified_by_unrefinement(
+          ownedParentElementsModifiedByUnrefinement, childElementsToDeleteForUnrefinement);
+    }
 
     if(stk::is_true_on_any_proc(mesh.parallel(), !childElementsToDeleteForUnrefinement.empty()))
     {
-
       for (auto parentElement : ownedParentElementsModifiedByUnrefinement)
       {
+        stk::diag::TimeBlock timer_2(unrefineTimer.restrictElementFields);
+
         auto childElements = get_children(parentElement);
         restrict_element_fields(mesh, parentElement, childElements);
       }
 
+      std::vector<int> originatingProcForParentsBeingModified;
       didMakeAnyChanges = true;
+{
+        stk::diag::TimeBlock timer_2(unrefineTimer.fillElements);
 
-      const std::vector<int> originatingProcForParentsBeingModified = get_originating_procs_for_elements(ownedParentElementsModifiedByUnrefinement);
+       originatingProcForParentsBeingModified =
+          get_originating_procs_for_elements(ownedParentElementsModifiedByUnrefinement);
+}
+      {
+        stk::diag::TimeBlock timer_2(unrefineTimer.meshMod);
 
-      mesh.modification_begin();
-      destroy_custom_ghostings();
-      stk::mesh::destroy_elements_no_mod_cycle(mesh, childElementsToDeleteForUnrefinement, mesh.mesh_meta_data().universal_part());
-      remove_parent_parts(ownedParentElementsModifiedByUnrefinement);
-      mesh.modification_end();
+        mesh.modification_begin();
+        destroy_custom_ghostings();
+        stk::mesh::destroy_elements_no_mod_cycle(
+            mesh, childElementsToDeleteForUnrefinement, mesh.mesh_meta_data().universal_part());
+        remove_parent_parts(ownedParentElementsModifiedByUnrefinement);
+        mesh.modification_end();
+      }
 
-      fix_face_and_edge_ownership(mesh);
+      {
+        stk::diag::TimeBlock timer_2(unrefineTimer.fixFaceEdgeOwnership);
+        fix_face_and_edge_ownership(mesh);
+        mark_already_refined_edges();
+      }
 
-      mark_already_refined_edges();
 
-      create_another_layer_of_refined_elements_and_sides_to_eliminate_hanging_nodes(edgeMarker);
+      {
+        stk::diag::TimeBlock timer_2(unrefineTimer.elminateHangingNodes);
+        create_another_layer_of_refined_elements_and_sides_to_eliminate_hanging_nodes(edgeMarker);
+      }
 
-      respect_originating_proc_for_parents_modified_by_unrefinement(ownedParentElementsModifiedByUnrefinement, originatingProcForParentsBeingModified);
+      {
+        stk::diag::TimeBlock timer_2(unrefineTimer.fixFaceEdgeOwnership);
+
+        respect_originating_proc_for_parents_modified_by_unrefinement(
+            ownedParentElementsModifiedByUnrefinement, originatingProcForParentsBeingModified);
+      }
     }
   }
 
-  check_leaf_children_have_parents_on_same_proc();
+  {
+    stk::diag::TimeBlock timer_2(unrefineTimer.checkLeafChildren);
+    check_leaf_children_have_parents_on_same_proc();
+  }
 
   return didMakeAnyChanges;
 }
 
 bool Refinement::do_refinement(const EdgeMarkerInterface & edgeMarker)
 {
-  stk::mesh::BulkData & mesh = myMeta.mesh_bulk_data();
-
-  check_leaf_children_have_parents_on_same_proc();
-
-  find_edges_to_refine(edgeMarker);
-
   bool didMakeAnyChanges = false;
 
-  const bool haveEdgesToRefineLocally = get_num_edges_to_refine() > 0;
-  if(stk::is_true_on_any_proc(mesh.parallel(), haveEdgesToRefineLocally))
-  {
-    didMakeAnyChanges = true;
-
-    create_refined_nodes_elements_and_sides(edgeMarker);
-
-    create_another_layer_of_refined_elements_and_sides_to_eliminate_hanging_nodes(edgeMarker);
-
-    myNodeRefiner.prolong_refined_edge_nodes(mesh);
-  }
-
-  didMakeAnyChanges |= do_unrefinement(edgeMarker);
+  didMakeAnyChanges |= refine_elements(edgeMarker);
+  didMakeAnyChanges |= unrefine_elements(edgeMarker);
 
   if (didMakeAnyChanges && myActivePart)
   {
-      activate_selected_entities_touching_active_elements(mesh, myMeta.side_rank(), myMeta.universal_part(), *myActivePart);
-      fix_node_owners_to_assure_active_owned_element_for_node(mesh, *myActivePart);
+    finalize();
   }
 
   STK_ThrowAssertMsg(!have_any_hanging_refined_nodes(), "Mesh has hanging refined node.");
