@@ -730,6 +730,131 @@ TEUCHOS_UNIT_TEST_TEMPLATE_3_DECL(
   }
 }
 
+TEUCHOS_UNIT_TEST_TEMPLATE_3_DECL(
+  Kokkos_View_Fad, LocalDeepCopy, FadType, Layout, Device )
+{
+  typedef Kokkos::View<FadType***,Layout,Device> ViewType;
+  typedef Kokkos::View<FadType,Layout,Device> ScalarViewType;
+  typedef typename ViewType::size_type size_type;
+  typedef typename ViewType::HostMirror host_view_type;
+  typedef typename ScalarViewType::HostMirror host_scalar_view_type;
+
+  const size_type num_rows = global_num_rows;
+  const size_type num_cols = global_num_cols;
+  const size_type num_slices = 10;
+  const size_type fad_size = global_fad_size;
+
+  // Create and fill view
+  ViewType v;
+#if defined (SACADO_DISABLE_FAD_VIEW_SPEC)
+  v = ViewType ("view", num_rows, num_cols, num_slices);
+#else
+  v = ViewType ("view", num_rows, num_cols, num_slices, fad_size+1);
+#endif
+  typename ViewType::array_type va = v;
+  Kokkos::deep_copy( va, 1.0 );
+
+  // Deep copy a constant Fad to the device
+  // Can't deep_copy directly because that doesn't work with DFad
+  FadType a(fad_size, 2.3456);
+  for (size_type i=0; i<fad_size; ++i)
+    a.fastAccessDx(i) = 7.89 + (i+1);
+#if defined (SACADO_DISABLE_FAD_VIEW_SPEC)
+  ScalarViewType a_view("a");
+#else
+  ScalarViewType a_view("a", fad_size+1);
+#endif
+  host_scalar_view_type ha_view = Kokkos::create_mirror_view(a_view);
+  ha_view() = a;
+  Kokkos::deep_copy( a_view, ha_view );
+
+  // Excersize local_deep_copy by setting each row of s to a
+  Kokkos::parallel_for(Kokkos::RangePolicy<Device>(0,num_rows),
+                       KOKKOS_LAMBDA(const int i)
+  {
+    auto s = Kokkos::subview(v,i,Kokkos::ALL,Kokkos::ALL);
+    Kokkos::Experimental::local_deep_copy(s,a_view());
+  });
+
+  // Copy back to host
+  host_view_type hv = Kokkos::create_mirror_view(v);
+  Kokkos::deep_copy(hv, a);
+
+  // Check
+  success = true;
+  for (size_type i=0; i<num_rows; ++i) {
+    for (size_type j=0; j<num_cols; ++j) {
+      for (size_type k=0; k<num_slices; ++k) {
+        success = success && checkFads(a, hv(i,j,k), out);
+      }
+    }
+  }
+}
+
+TEUCHOS_UNIT_TEST_TEMPLATE_3_DECL(
+  Kokkos_View_Fad, LocalDeepCopyTeam, FadType, Layout, Device )
+{
+  typedef Kokkos::View<FadType***,Layout,Device> ViewType;
+  typedef Kokkos::View<FadType,Layout,Device> ScalarViewType;
+  typedef typename ViewType::size_type size_type;
+  typedef typename ViewType::HostMirror host_view_type;
+  typedef typename ScalarViewType::HostMirror host_scalar_view_type;
+
+  const size_type num_rows = global_num_rows;
+  const size_type num_cols = global_num_cols;
+  const size_type num_slices = 10;
+  const size_type fad_size = global_fad_size;
+
+  // Create and fill view
+  ViewType v;
+#if defined (SACADO_DISABLE_FAD_VIEW_SPEC)
+  v = ViewType ("view", num_rows, num_cols, num_slices);
+#else
+  v = ViewType ("view", num_rows, num_cols, num_slices, fad_size+1);
+#endif
+  typename ViewType::array_type va = v;
+  Kokkos::deep_copy( va, 1.0 );
+
+  // Deep copy a constant Fad to the device
+  // Can't deep_copy directly because that doesn't work with DFad
+  FadType a(fad_size, 2.3456);
+  for (size_type i=0; i<fad_size; ++i)
+    a.fastAccessDx(i) = 7.89 + (i+1);
+#if defined (SACADO_DISABLE_FAD_VIEW_SPEC)
+  ScalarViewType a_view("a");
+#else
+  ScalarViewType a_view("a", fad_size+1);
+#endif
+  host_scalar_view_type ha_view = Kokkos::create_mirror_view(a_view);
+  ha_view() = a;
+  Kokkos::deep_copy( a_view, ha_view );
+
+  // Excersize local_deep_copy by setting each row of s to a
+  typedef Kokkos::TeamPolicy<Device> Policy;
+  static const size_type stride = Kokkos::ViewScalarStride<ViewType>::stride;
+  Kokkos::parallel_for(Policy(num_rows,Kokkos::AUTO,stride),
+                       KOKKOS_LAMBDA(const typename Policy::member_type& team)
+  {
+    int i = team.league_rank();
+    auto s = Kokkos::subview(v,i,Kokkos::ALL,Kokkos::ALL);
+    Kokkos::Experimental::local_deep_copy(team,s,a_view());
+  });
+
+  // Copy back to host
+  host_view_type hv = Kokkos::create_mirror_view(v);
+  Kokkos::deep_copy(hv, a);
+
+  // Check
+  success = true;
+  for (size_type i=0; i<num_rows; ++i) {
+    for (size_type j=0; j<num_cols; ++j) {
+      for (size_type k=0; k<num_slices; ++k) {
+        success = success && checkFads(a, hv(i,j,k), out);
+      }
+    }
+  }
+}
+
 TEUCHOS_UNIT_TEST_TEMPLATE_3_DECL(
   Kokkos_View_Fad, ScalarAssign, FadType, Layout, Device )
 {
@@ -2464,6 +2589,8 @@ TEUCHOS_UNIT_TEST_TEMPLATE_3_DECL(
   TEUCHOS_UNIT_TEST_TEMPLATE_3_INSTANT( Kokkos_View_Fad, DeepCopy_ConstantZero, F, L, D ) \
   TEUCHOS_UNIT_TEST_TEMPLATE_3_INSTANT( Kokkos_View_Fad, DeepCopy_ConstantFad, F, L, D ) \
   TEUCHOS_UNIT_TEST_TEMPLATE_3_INSTANT( Kokkos_View_Fad, DeepCopy_ConstantFadFull, F, L, D ) \
+  TEUCHOS_UNIT_TEST_TEMPLATE_3_INSTANT( Kokkos_View_Fad, LocalDeepCopy, F, L, D ) \
+  TEUCHOS_UNIT_TEST_TEMPLATE_3_INSTANT( Kokkos_View_Fad, LocalDeepCopyTeam, F, L, D ) \
   TEUCHOS_UNIT_TEST_TEMPLATE_3_INSTANT( Kokkos_View_Fad, ScalarAssign, F, L, D ) \
   TEUCHOS_UNIT_TEST_TEMPLATE_3_INSTANT( Kokkos_View_Fad, ValueAssign, F, L, D ) \
   TEUCHOS_UNIT_TEST_TEMPLATE_3_INSTANT( Kokkos_View_Fad, Resize, F, L, D ) \
