@@ -435,7 +435,7 @@ public:
         const ordinal_type used_span = max(ABR.span(), T.span());
         UnmanagedViewType<value_type_array> W(bufptr + used_span, int(bufend - bufbeg - used_span));
         factorize_var1(member, s, P, D, W, T, ABR);
-      } else if (factorize_tag_type::variant == 2) {
+      } else if (factorize_tag_type::variant == 2 || factorize_tag_type::variant == 3) {
         const ordinal_type bufbeg = _buf_ptr(lid), bufend = _buf_ptr(lid + 1);
         auto bufptr = _buf.data() + bufbeg;
         UnmanagedViewType<value_type_matrix> ABR(bufptr, n_m, n_m);
