@@ -433,7 +433,7 @@ template<class Scalar>
 inline
 void field_axpy(const Scalar alpha, const FieldBase& xField, const FieldBase& yField, const Selector& selector)
 {
-    STK_ThrowAssert(is_compatible(xField, yField));
+    STK_ThrowRequire(is_compatible(xField, yField));
 
     BucketVector const& buckets = xField.get_mesh().get_buckets( xField.entity_rank(), selector );
 
@@ -463,7 +463,7 @@ template<class Scalar>
 inline
 void field_axpby(const Scalar alpha, const FieldBase& xField, const Scalar beta, const FieldBase& yField, const Selector& selector)
 {
-    STK_ThrowAssert(is_compatible(xField, yField));
+    STK_ThrowRequire(is_compatible(xField, yField));
 
     BucketVector const& buckets = xField.get_mesh().get_buckets( xField.entity_rank(), selector );
 
@@ -524,8 +524,8 @@ void INTERNAL_field_product(const FieldBase& xField, const FieldBase& yField, co
 inline
 void field_product(const FieldBase& xField, const FieldBase& yField, const FieldBase& zField, const Selector& selector)
 {
-    STK_ThrowAssert(is_compatible(xField, yField));
-    STK_ThrowAssert(is_compatible(yField, zField));
+    STK_ThrowRequire(is_compatible(xField, yField));
+    STK_ThrowRequire(is_compatible(yField, zField));
 
     if (xField.data_traits().type_info == typeid(double)) {
         INTERNAL_field_product<double>(xField,yField,zField,selector);
@@ -594,7 +594,7 @@ void INTERNAL_field_copy(const FieldBase& xField, const FieldBase& yField, const
 inline
 void field_copy(const FieldBase& xField, const FieldBase& yField, const Selector& selector)
 {
-    STK_ThrowAssert(is_compatible(xField, yField));
+    STK_ThrowRequire(is_compatible(xField, yField));
 
     if (xField.data_traits().type_info == typeid(double)) {
         INTERNAL_field_copy<double>(xField,yField,selector);
@@ -622,7 +622,7 @@ template<class Scalar,class T1,class T2,class T3,class T4,class T5,class T6,clas
 inline
 Scalar field_dot(const Field<Scalar,T1,T2,T3,T4,T5,T6,T7> & xField, const Field<Scalar,T1,T2,T3,T4,T5,T6,T7> & yField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert(is_compatible(xField, yField));
+    STK_ThrowRequire(is_compatible(xField, yField));
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -650,7 +650,7 @@ template<class Scalar,class T1,class T2,class T3,class T4,class T5,class T6,clas
 inline
 std::complex<Scalar> field_dot(const Field<std::complex<Scalar>,T1,T2,T3,T4,T5,T6,T7>& xField, const Field<std::complex<Scalar>,T1,T2,T3,T4,T5,T6,T7>& yField, const Selector& selector, const MPI_Comm comm) 
 {
-    STK_ThrowAssert(is_compatible(xField, yField));
+    STK_ThrowRequire(is_compatible(xField, yField));
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -700,7 +700,7 @@ template<class Scalar>
 inline
 void field_dot(std::complex<Scalar>& global_result, const FieldBase& xField, const FieldBase& yField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert(is_compatible(xField, yField));
+    STK_ThrowRequire(is_compatible(xField, yField));
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -734,7 +734,7 @@ template<class Scalar>
 inline
 void field_dot(Scalar& glob_result, const FieldBase& xField, const FieldBase& yField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert(is_compatible(xField, yField));
+    STK_ThrowRequire(is_compatible(xField, yField));
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -777,7 +777,7 @@ template<class Scalar>
 inline
 void field_scale(const Scalar alpha, const FieldBase& xField, const Selector& selector)
 {
-    STK_ThrowAssert( is_compatible<Scalar>(xField) );
+    STK_ThrowRequire( is_compatible<Scalar>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(),selector);
 
@@ -804,7 +804,7 @@ template<class Scalar>
 inline
 void field_fill_component(const Scalar* alpha, const FieldBase& xField, const Selector& selector)
 {
-    STK_ThrowAssert( is_compatible<Scalar>(xField) );
+    STK_ThrowRequire( is_compatible<Scalar>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(),selector);
 
@@ -837,7 +837,7 @@ template<class Scalar>
 inline
 void field_fill(const Scalar alpha, const FieldBase& xField, const Selector& selector)
 {
-    STK_ThrowAssert( is_compatible<Scalar>(xField) );
+    STK_ThrowRequire( is_compatible<Scalar>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(),selector);
 
@@ -856,7 +856,7 @@ template<class Scalar>
 inline
 void field_fill(const Scalar alpha, const std::vector<const FieldBase*>& xFields, const Selector& selector)
 {
-    STK_ThrowAssert(xFields.size() >= 1 );
+    STK_ThrowRequire(xFields.size() >= 1 );
 
     stk::mesh::EntityRank fieldEntityRank = xFields[0]->entity_rank();
     BucketVector const& buckets = xFields[0]->get_mesh().get_buckets(fieldEntityRank,selector);
@@ -864,8 +864,8 @@ void field_fill(const Scalar alpha, const std::vector<const FieldBase*>& xFields
     for (auto&& bucket : buckets){
         for (unsigned int i=0; i<xFields.size(); ++i){
             const FieldBase& xField = *xFields[i];
-            STK_ThrowAssert( is_compatible<Scalar>(xField) );
-            STK_ThrowAssert(fieldEntityRank == xField.entity_rank());
+            STK_ThrowRequire( is_compatible<Scalar>(xField) );
+            STK_ThrowRequire(fieldEntityRank == xField.entity_rank());
             BucketSpan<Scalar> x(xField, *bucket);
             FortranBLAS<Scalar>::fill(x.size(),alpha,x.data());
         }
@@ -911,7 +911,7 @@ void INTERNAL_field_swap(const FieldBase& xField, const FieldBase& yField, const
 inline
 void field_swap(const FieldBase& xField, const FieldBase& yField, const Selector& selector)
 {
-    STK_ThrowAssert(is_compatible(xField, yField));
+    STK_ThrowRequire(is_compatible(xField, yField));
 
     if (xField.data_traits().type_info == typeid(double)) {
         INTERNAL_field_swap<double>(xField,yField,selector);
@@ -1001,7 +1001,7 @@ template<class Scalar>
 inline
 void field_nrm2(Scalar& glob_result, const FieldBase& xField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert( is_compatible<Scalar>(xField) );
+    STK_ThrowRequire( is_compatible<Scalar>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -1026,7 +1026,7 @@ template<class Scalar>
 inline
 void field_nrm2(std::complex<Scalar>& result, const FieldBase& xField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert( is_compatible<std::complex<Scalar>>(xField) );
+    STK_ThrowRequire( is_compatible<std::complex<Scalar>>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -1129,7 +1129,7 @@ template<class Scalar>
 inline
 void field_asum(Scalar& glob_result, const FieldBase& xField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert( is_compatible<Scalar>(xField) );
+    STK_ThrowRequire( is_compatible<Scalar>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -1153,7 +1153,7 @@ template<class Scalar>
 inline
 void field_asum(std::complex<Scalar>& result, const FieldBase& xField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert( is_compatible<std::complex<Scalar>>(xField) );
+    STK_ThrowRequire( is_compatible<std::complex<Scalar>>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -1408,7 +1408,7 @@ template<class Scalar>
 inline
 void field_amax(std::complex<Scalar>& result, const FieldBase& xField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert( is_compatible<std::complex<Scalar>>(xField) );
+    STK_ThrowRequire( is_compatible<std::complex<Scalar>>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -1438,7 +1438,7 @@ template<class Scalar>
 inline
 void field_amax(Scalar& result, const FieldBase& xField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert( is_compatible<Scalar>(xField) );
+    STK_ThrowRequire( is_compatible<Scalar>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -1809,7 +1809,7 @@ template<class Scalar>
 inline
 void field_amin(std::complex<Scalar>& result, const FieldBase& xField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert( is_compatible<std::complex<Scalar>>(xField) );
+    STK_ThrowRequire( is_compatible<std::complex<Scalar>>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
@@ -1839,7 +1839,7 @@ template<class Scalar>
 inline
 void field_amin(Scalar& result, const FieldBase& xField, const Selector& selector, const MPI_Comm comm)
 {
-    STK_ThrowAssert( is_compatible<Scalar>(xField) );
+    STK_ThrowRequire( is_compatible<Scalar>(xField) );
 
     BucketVector const& buckets = xField.get_mesh().get_buckets(xField.entity_rank(), selector & xField.mesh_meta_data().locally_owned_part());
 
